function obj=deflate(varargin)
% deflate - constructor for deflated matrix
% 
% A deflated matrix is used for storage of large non-sparse N-D matrices of
% up to 2^32 elements.
% 
% CALL
%     m=deflate(M), OR:
%     m=deflate(M,<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>), OR:
%     m=deflate(<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..)
%
% INPUT
%     M      : numeric or char matrix, deflated matrix, or structure with
%              equivalent fields
% 
%     <PROPERTY>,<VALUE>: property value pair. 
%     (*): passed on to deflatev and not used in deflate
%          Valid combinations are:
%     'size':    size of deflated matrix                
%                Only allowed when if argument "M" is not available.
%                Defaults to size(M) or [0,0] if M is not specified
%  (*)'emptyvalue': 
%                This value will be assumed for all elements that are not
%                specified.
%                Only allowed when if argument "M" is not available.
%                Defaults to double(0). 
%  (*)'mode'   : Storage mode: memory, compressed, disk
%                Defaults to compressed
%                compressed: equal to memory, but now the data blocks are
%                        compressed in memory. This expands the maximum 
%                        storage size considerably. If random access is
%                        required the CPU costs may be high.
%                memory: data is stored in memory in contiguous blocks of
%                        size "segmentSize". This expands the maximum
%                        storage size typically with a factor 3. compared
%                        to storage in contiguous blocks. The CPU costs are
%                        moderate.
%                disk:   TO BE IMPLEMENTED the data blocks are stored on disk. This is faster
%                        then compression, but also more vulnarable. Copies
%                        of the object point to the same file, unless a
%                        "Deep" copy is made
%  (*)'segmentSize': smallest blocksize to break up vector
%                    Defaults to 100000
% 
% OUTPUT
%     m: deflated matrix corresponding to M
%   
% INTERNAL DATASTRUCTURE
%     m
%     +----size      : size of matrix stored
%     +----data      : deflatev object holding elements
% 
% EXAMPLE (1)
%     M=deflate('class','int16','size',[1000000,170]);
%     for k=1:size(M,2)
%         disp(k)
%         M(:,k)=int16(1:size(M,1));
%     end
% 
% EXAMPLE (2)
%     Verify methods "constructor", "struct", and "full"
%     a=rand(1000);is_eq(a,full(deflate(struct(deflate(a)))));
    
%-----------------------
%No arguments
%-----------------------
if nargin==0
    obj=deflate([]);
    return
end


%-----------------------
%1 or more arguments
%-----------------------
% Special case
% -1- implement deflate(obj), with obj of class deflate
if isa(varargin{1},'deflate')
    obj=varargin{1};
    if nargin>1
        error('When first argument is deflate object extra arguments are not allowed');
    end
    return
end
% Special case
% -2- implement deflate(S), with S a structure converted from a deflate object
if isstruct(varargin{1}) && nargin==1
    %make sure that all fields are present:
    if isfield(varargin{1},'size')&&isfield(varargin{1},'data')
        %all required fileds are there
        S=varargin{1};
        S.data=deflatev(varargin{1}.data);
        obj=class(S,'deflate');
        return
    end
end    

%-----------------------------------------------------------
%1 or more arguments first argument is not deflate object
%-----------------------------------------------------------
if rem(nargin,2)==1
    %The first argument contains the data
    %M implies size. emptyvalue is not relevant.
    M=varargin{1};
    argused=1;
    ValidProps={'mode','segmentSize','emptyvalue'};
else
    %create a deflate matrix with zeros
    M         =[];
    argused   =0;
    ValidProps={'mode','segmentSize','emptyvalue','size'};
end

%-----------------------------------------------------------
%First argument in interpreted, check remaining arguments
%-----------------------------------------------------------

% Note: 
% The objective here is to dermine the "size" property
% The constructor "deflatev" takes care of all other input arguments
   
%-----------------------------------------------------------
%Read options
%-----------------------------------------------------------

%specify default for attribute "size"
opt.size         = size(M);

%WIJZ ZIJPP 20100225: when possible, use opt.size(1) as segmentsize
if opt.size(1)>1000 && opt.size(1)<10^6
    opt.segmentSize=opt.size(1);
else
    
end
%overwrite size argument if it was specified
opt=varargin2struct(opt,ValidProps,varargin{argused+1:end});
S.size  =  opt.size;

%pass all other arguments through to deflatev (argument "size" will be ignored)
if rem(nargin,2)==1
    %first argument contains data
    S.data  = deflatev(varargin{:});
else
    %data vector not specified, pass on "N" instead
    S.data  = deflatev(varargin{:},'N',prod(opt.size));
end


%-----------------------------------------------------------
%Initialize object
%-----------------------------------------------------------
obj=class(S,'deflate');







