function [subscript,OOB]=getSubscript(ind,sz)
% getSubscript - convert SUBSTRUCT subscript containing ':' to
%                cell array of indices
%
% INPUT
%     ind: general SUBSTRUCT index
%     sz : size of object
%
% OUTPUT
%     subscript: cell array of numeric index arrays
%                example: a(:,2:3) ==> subscript = {1:size(a,1),2:3}
%     OOB      : Out Of Bound flag
%                true : assignment/reference out of bound
%                       (not allowed in refernce, allowed in assignment)
%                false: assignment/reference within bounds
%
% OPTIONS
%     -1- number of subsscripts == number of dimensions
%     -2- number of subscripts  <  number of dimensions
%         Example: a=rand(2,3); a(1)..a(6) refer to valid dimension
%
%     -3- number of subscripts  >  number of dimensions
%         Example: a=rand(2,3); a(2,3,2)=1.
%                  The number of dimensions of a will grow
%         Example: a=rand(2,3); a(2,3,1)=1.
%                  The number of dimensions of a will not grow

%Check syntax
%The expected assignment/reference is:
%     ARRAY(I1,I2,...)
%The corresponding substruct is:
%     struct('type','()','subs',{[I1],[I2],...})
%     NOT ALLOWED: M_deflate(2).field
if ind(1).type(1)~='(' || length(ind)>1
    error('Subscripts in compressed matrix should be of form: M(I1,I2,I2,...)');
end
OOB=false;
subscript=ind.subs;
if length(subscript)==length(sz)
    %This is the most common case.
    %- the assignment does not change the number of dimensions
    %- for each dimension an index is specified
    [subscript,OOB]=replaceCol(subscript,sz);
    return
end

if length(subscript)<length(sz)
    %Exceptional case.
    %- the assignment does not change the number of dimensions
    %- index not specified for 1 or more dimensions
    %- this should be treated as a linear index

    %WIJZ ZIJPP FEB 2008: if last index is not ":", append 1 for all missing indices
    if ~strcmp(subscript{end},':')
        %last index is not ":"
        [subscript,OOB]=replaceCol(subscript,sz);
        %append 1 for missing subsripts
        for k=length(subscript)+1:length(sz)
            subscript{k}=1;
        end
        return
    end

    %Replace ':' with numbers
    for k=1:length(subscript)
        if strcmp(subscript{k},':')
            %if this is the last specified dimension: assume (:) for all next
            %dimensions
            if k==length(subscript)
                subscript{k}=1:prod(sz(k:end));
            else
                subscript{k}=1:sz(k);
            end
        end
    end
    %convert to linear index
    linIndices=sub2indVector(sz,subscript{:});
    subscript = {linIndices};
    if max(linIndices)>prod(sz)
        OOB=true;
    end
    return
end

%ALL OTHER CASES: length(subscript)>length(sz)
%Extend either subscript of size with ones

%ignore ones at end
for k=length(subscript):-1:(length(sz)+1) %WIJZ ZIJPP FEB 2008: add step=-1
    if subscript{k}~=1
        break
    end
    subscript(k)=[];
end

% if number of indices is higher then number of dimensions of array, assume
% 1 for missing dimensions (this may or may not lead to OOB warning)
% extend size vector with singleton dimensions if needed
for k=(length(sz)+1):length(subscript)
    sz(k)=1;
end

%subscript and size have again equal size
[subscript,OOB]=replaceCol(subscript,sz);
%__________________________________________________________________________
function [subscript,OOB]=replaceCol(subscript,sz)
% replace ':' with numeric equivalent
OOB=false;
for k=1:length(subscript)
    if strcmp(subscript{k},':')
        %if this is the last specified dimension: assume (:) for all next
        %dimensions
        subscript{k}=1:sz(k);
    else
        if max(subscript{k})>sz(k) %Nog te checken!
            OOB=true;
        end
    end
end

